/*
 * The Syncro Soft SRL License
 * 
 * 
 * Copyright (c) 1998-2002 Syncro Soft SRL, Romania. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met: 1. Redistribution of source or in binary form is
 * allowed only with the prior written permission of Syncro Soft SRL. 2. Redistributions of source
 * code must retain the above copyright notice, this list of conditions and the following
 * disclaimer. 2. Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution. 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment: "This product includes software developed by
 * the Syncro Soft SRL (http://www.sync.ro/)." Alternately, this acknowledgment may appear in the
 * software itself, if and wherever such third-party acknowledgments normally appear. 4. The names
 * "Oxygen" and "Syncro Soft SRL" must not be used to endorse or promote products derived from this
 * software without prior written permission. For written permission, please contact
 * contact@sync.ro. 5. Products derived from this software may not be called "Oxygen", nor may
 * "Oxygen" appear in their name, without prior written permission of the Syncro Soft SRL.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oxygenxml.editor.sample.extension.action;

import javax.swing.text.BadLocationException;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;

import com.oxygenxml.editor.editors.xml.extension.OutlineExtensionAction;
import com.oxygenxml.editor.sample.extension.SamplePlugin;

/**
 * Action to be added to outline context menu for selecting just the 
 * start tag corresponding to the current selected node in the outline tree.
 * <p>
 * The oXygen plugin will set in the member tokenItem the current selection
 * of the outline tree and in the member editor the current Eclipse editor. 
 * 
 * @version $Revision: 1.1 $
 */
public class SelectTagAction extends OutlineExtensionAction implements IExecutableExtension {

  /**
   * Logger for logging.
   */
  private static Logger logger = Logger.getLogger(SelectTagAction.class.getName());
  
  /**
   * Name of attribute in plugin.xml with menu item name.
   */
  private final static String OUTLINE_MENU_ITEM_NAME_ATTRIB_NAME = "label";

  /**
   * Name of attribute in plugin.xml with menu item icon.
   */
  private final static String OUTLINE_MENU_ITEM_ICON_ATTRIB_NAME = "icon";

  /**
   * Sets the extension parameters.
   * 
   * @param configElement The configuration element.
   * @param attrName The attribute name.
   * @param initData The init data.
   * @throws CoreException when the init fails.
   */
  public void setInitializationData(IConfigurationElement configElement, String attrName, Object initData)
    throws CoreException {
    setText(configElement.getAttribute(OUTLINE_MENU_ITEM_NAME_ATTRIB_NAME));
    String iconName = configElement.getAttribute(OUTLINE_MENU_ITEM_ICON_ATTRIB_NAME);
    setImageDescriptor(SamplePlugin.imageDescriptorFromPlugin(SamplePlugin.PLUGIN_ID, iconName));
  }

  /**
   * Compute the row and column of the outline tree selection and select the start tag of 
   * this selection.
   */
  public void run() {
    logger.info("begin run");
    if (tokenItem != null) {
      int row = tokenItem.getLine();
      int column = tokenItem.getColumn();
      logger.info("row column: " + row + "  " + column);
      try {
        logger.info("document: " + document.getLineCount());
        int start = document.getOffsetOfLineStart(row + 1) + column;
        logger.info("start: " + start);
        editor.selectAndReveal(start, tokenItem.getLength());
        logger.info("revealed: " + start + "  " + tokenItem.getLength());
      } catch (BadLocationException e) {
        logger.error(e, e);
      } catch (Throwable t) {
        logger.error("===== " + t, t);
      }
      logger.info("end run");
    }
  }
}
