/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.css;

import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.util.Equaler;

/**
 * Contains counter information: name, list style.
 */
@API(type=APIType.INTERNAL, src=SourceType.PRIVATE)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.NOT_SPECIFIED, methods = SkipLevel.PUBLIC)
public class CounterContent implements StaticContent {
  
  /**
   * The name of the counter.
   */
  private String counterName;

  /**
   * The list style type.
   */
  private String listStyleType = CSS.DECIMAL;
  
  /**
   * Constructor
   * 
   * @param cName The name of the counter.
   */
  public CounterContent(String cName) {
    this.counterName = cName;
  }
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    int hash = 0;
    if (counterName != null) {
      hash = counterName.intern().hashCode();
    }
    if (listStyleType != null) {
      hash += listStyleType.intern().hashCode();
    }
    return hash;
  }
  
  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof CounterContent) {
      CounterContent cc = (CounterContent) obj;
      return Equaler.verifyEquals(counterName, cc.counterName) && 
          Equaler.verifyEquals(listStyleType, cc.listStyleType);
    }
    return false;
  }

  /**
   * @see ro.sync.ecss.css.StaticContent#getType()
   */
  public int getType() {
    return COUNTER_CONTENT;
  }

  /**
   * @return The name of the counter.
   */
  public String getCounterName() {
    return counterName;
  }

  /**
   * @return The list style type of the counter.
   */
  public String getListStyleType() {
    return listStyleType;
  }

  /**
   * Sets the list style type.
   * 
   * @param styleType The style type.
   */
  public void setListStyleType(String styleType) {
    this.listStyleType = styleType;
  }
}
