/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2011 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.workspace.api.standalone.InputURLChooserCustomizer;
import ro.sync.exml.workspace.api.standalone.ReferencesCustomizer;
import ro.sync.exml.workspace.api.util.RelativeReferenceResolver;
import ro.sync.exml.workspace.impl.standalone.RelativeReferenceResolversProvider;
import ro.sync.exml.workspace.impl.standalone.URLChooserCustomizersProvider;

/**
 * Customizers for the author component factory.
 */
@API(type=APIType.INTERNAL, src=SourceType.PRIVATE)
public class AuthorComponentFactoryCustomizers implements ReferencesCustomizer, RelativeReferenceResolversProvider, URLChooserCustomizersProvider{

  /**
   * The map of resolvers grouped by protocol
   */
  private Map<String, List<RelativeReferenceResolver>> relativeReferenceResolvers = null;
  
  /**
   * The list of URL chooser customizers.
   */
  private List<InputURLChooserCustomizer> urlChooserCustomizers = new ArrayList<InputURLChooserCustomizer>();

  /**
   * @see ro.sync.exml.workspace.api.standalone.ReferencesCustomizer#addRelativeReferencesResolver(java.lang.String, ro.sync.exml.workspace.api.util.RelativeReferenceResolver)
   */
  public void addRelativeReferencesResolver(String protocol, RelativeReferenceResolver resolver) {
    if(relativeReferenceResolvers == null) {
      relativeReferenceResolvers = new HashMap<String, List<RelativeReferenceResolver>>();
    }
    List<RelativeReferenceResolver> resolversForProtocol = relativeReferenceResolvers.get(protocol);
    if(resolversForProtocol == null) {
      resolversForProtocol = new ArrayList<RelativeReferenceResolver>();
      relativeReferenceResolvers.put(protocol, resolversForProtocol);
    }
    resolversForProtocol.add(resolver);    
  }

  /**
   * @see ro.sync.exml.workspace.impl.standalone.RelativeReferenceResolversProvider#getRelativeReferenceResolvers(java.lang.String)
   */
  public List<RelativeReferenceResolver> getRelativeReferenceResolvers(String protocol) {
    return relativeReferenceResolvers == null ? null : relativeReferenceResolvers.get(protocol);
  }

  /**
   * @see ro.sync.exml.workspace.api.standalone.ReferencesCustomizer#addInputURLChooserCustomizer(ro.sync.exml.workspace.api.standalone.InputURLChooserCustomizer)
   */
  public void addInputURLChooserCustomizer(InputURLChooserCustomizer inputURLChooserCustomizer) {
    urlChooserCustomizers.add(inputURLChooserCustomizer);
  }
  
  /**
   * @return Returns the urlChooserCustomizers.
   */
  public List<InputURLChooserCustomizer> getUrlChooserCustomizers() {
    return urlChooserCustomizers;
  }
}
