/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2011 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;


import java.io.File;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.db.SQLTableModel;
import ro.sync.diff.ui.DiffFilesPresenter;
import ro.sync.exml.PreferencePagePresenter;
import ro.sync.exml.SimpleSAIDEAccess;
import ro.sync.exml.editor.ResourceManagerFactory;
import ro.sync.exml.editor.identifiers.Identifier;
import ro.sync.exml.editor.xmleditor.transform.ant.ANTTransformationHelperInterface;
import ro.sync.exml.editor.xmleditor.transform.dita.DITATransformationHelperInterface;
import ro.sync.exml.editor.xproc.transform.XProcTransformationHelperInterface;
import ro.sync.exml.masterfiles.IMasterFilesSupport;
import ro.sync.exml.masterfiles.MasterFilesSupportAdapter;
import ro.sync.exml.modules.core.ModuleType;
import ro.sync.exml.project.ProjectRefreshFilesEvent;

/**
 * Author Component IDE Access.
 */
@API(type=APIType.INTERNAL, src=SourceType.PRIVATE)
public class AuthorComponentIDEAccess extends SimpleSAIDEAccess {
  
  /**
   * Master files support.
   */
  private IMasterFilesSupport masterFilesSupport;

  /**
   * @see ro.sync.exml.BasicIDEAccessInterface#getPreferencePagePresenter()
   */
  @Override
  public PreferencePagePresenter getPreferencePagePresenter() {
    // No preference page for Author Component.
    return null;
  }
  /**
   * @see ro.sync.exml.IDEAccess#updateResults(java.lang.String, ro.sync.db.SQLTableModel)
   */
  @Override
  public void updateResults(String key, SQLTableModel tableModel) {
    // Do nothing.
  }
  /**
   * @see ro.sync.exml.IDEAccess#showHierarchyOrDependences(java.lang.String, boolean, java.lang.String)
   */
  @Override
  public void showHierarchyOrDependences(String systemId, boolean showDependences,
      String contentType) {
    // Do nothing.
  }
  /**
   * @see ro.sync.exml.IDEAccess#showDockableView(java.lang.String, boolean)
   */
  @Override
  public void showDockableView(String viewID, boolean requestFocus) {
    // Do nothing.
  }
  /**
   * @see ro.sync.exml.IDEAccess#showComponentDependencies(ro.sync.exml.editor.identifiers.Identifier, ro.sync.exml.editor.finders.SearchReferencesConstants.ModuleType)
   */
  @Override
  public void showComponentDependencies(Identifier identifier, ModuleType[] resourceType) {
    // Do nothing.
  }
  /**
   * @see ro.sync.exml.IDEAccess#addWatchExpression(java.lang.String)
   */
  @Override
  public void addWatchExpression(String watchExpression) {
    // Do nothing.
  }
  /**
   * @see ro.sync.exml.IDEAccess#getXProcTransformationHelper()
   */
  @Override
  public XProcTransformationHelperInterface getXProcTransformationHelper() {
    // No XProc transformation helper for Author Component.
    return null;
  }
  /**
   * @see ro.sync.exml.IDEAccess#getResourceManagerFactory()
   */
  @Override
  public ResourceManagerFactory getResourceManagerFactory() {
    // No resource manager factory for Author Component.
    return null;
  }
  /**
   * @see ro.sync.exml.IDEAccess#getDiffFilePresenter()
   */
  @Override
  public DiffFilesPresenter getDiffFilePresenter() {
    // No diff files presenter for Author Component.
    return null;
  }
  /**
   * @see ro.sync.exml.IDEAccess#getDITATransformationHelper()
   */
  @Override
  public DITATransformationHelperInterface getDITATransformationHelper() {
    // No DITA transformation helper for Author Component.
    return null;
  }
  /**
   * @see ro.sync.exml.IDEAccess#getANTTransformationHelper()
   */
  @Override
  public ANTTransformationHelperInterface getANTTransformationHelper() {
    // No ANT transformation helper for Author Component.
    return null;
  }
  
  /**
   * @see ro.sync.exml.IDEAccess#getMasterFilesSupport()
   */
  @Override
  public IMasterFilesSupport getMasterFilesSupport() {
    if(masterFilesSupport == null) {
      masterFilesSupport = new MasterFilesSupportAdapter();
    }
    return masterFilesSupport;
  }
  
  /**
   * @see ro.sync.exml.IDEAccess#refreshFilesInProject(ProjectRefreshFilesEvent)
   */
  @Override
  public void refreshFilesInProject(ProjectRefreshFilesEvent refreshFilesEvent) {
    // Nothing to be notified.
  }
  
  /**
   * @see ro.sync.exml.IDEAccess#doMoveResource(java.io.File, java.io.File, boolean)
   */
  @Override
  public void doMoveResource(File src, File dst, boolean move) {
    // No implementation for Author Component.
  }
}