/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.component;

import java.awt.Component;
import java.awt.Frame;
import java.io.File;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JToolBar;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.document.Descriptor;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.ExtensionsBundle;
import ro.sync.ecss.extensions.api.component.listeners.AuthorComponentListener;
import ro.sync.ecss.layout.DisplayModeConstants;
import ro.sync.ecss.persistance.AuthorExtensionPO;
import ro.sync.ecss.persistance.CSSDescriptorPO;
import ro.sync.exml.editor.AbstractEditor;
import ro.sync.exml.editor.BasicEditor;
import ro.sync.exml.editor.EditorListener;
import ro.sync.exml.editor.EditorPage;
import ro.sync.exml.editor.EditorPageChangeEvent;
import ro.sync.exml.editor.EditorPageConstants;
import ro.sync.exml.editor.FileDropHandler;
import ro.sync.exml.editor.PersistentEditor;
import ro.sync.exml.editor.persistance.DocumentTypePO;
import ro.sync.exml.editor.status.StatusPanel;
import ro.sync.exml.editor.xmleditor.DetectionTaskListener;
import ro.sync.exml.editor.xmleditor.DocumentTypeProviderListener;
import ro.sync.exml.editor.xmleditor.XmlEditor;
import ro.sync.exml.editor.xmleditor.pageauthor.AuthorEditorPage;
import ro.sync.exml.editor.xmleditor.pagegrid.GridEditorPage;
import ro.sync.exml.workspace.api.editor.WSEditor;
import ro.sync.exml.workspace.api.editor.page.author.WSAuthorComponentEditorPage;
import ro.sync.exml.workspace.api.editor.page.author.WSAuthorEditorPage;
import ro.sync.exml.workspace.api.editor.page.text.WSTextEditorPage;
import ro.sync.exml.workspace.api.listeners.WSEditorPageChangedListener;
import ro.sync.exml.workspace.impl.editor.WSEditorImpl;
import ro.sync.licensemanager.LicenseInformation;
import ro.sync.licensemanager.LicenseInformationProvider;
import ro.sync.ui.UiUtil;
import ro.sync.ui.application.action.AccelAction;
import ro.sync.util.Equaler;
import ro.sync.util.URLInfo;
import ro.sync.util.URLUtil;

/**
 * A component encapsulating all the visual editing part.
 * Developers can set the XML and CSS files, and access the document through the {@link AuthorAccess} API.
 *  
 * @author dan
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PRIVATE)
public class AuthorComponentProvider extends InternalComponentProvider implements DisplayModeConstants, EditorComponentProvider{
  /** 
   * Logger for logging. 
   */
  private static final Logger logger = Logger.getLogger(AuthorComponentProvider.class.getName());
  
  /**
   * The author component listeners
   */
  private final List<AuthorComponentListener> authorComponentListeners =
    new ArrayList<AuthorComponentListener>();

  /**
   * The editor.
   */
  private final XmlEditor editor;
  
  /**
   * The status panel
   */
  private final StatusPanel statusPanel = new StatusPanel(true, true);

  /**
   * True if the detection finished
   */
  protected boolean detectionFinished = false;
  
  /**
   * Outliner Panel
   */
  public static final int OUTLINER_PANEL_ID = 0;
  
  /**
   * Attributes Panel
   */
  public static final int ATTRIBUTES_PANEL_ID = 1;
  
  /**
   * Model Panel
   */
  public static final int MODEL_PANEL_ID = 2;
  
  /**
   * Elements Panel
   */
  public static final int ELEMENTS_PANEL_ID = 3;
  
  /**
   * Entities Panel
   */
  public static final int ENTITIES_PANEL_ID = 4;
  
  /**
   * Reviews Panel
   */
  public static final int REVIEWS_PANEL_ID = 5;
  
  /**
   * Get access to thw WS Editor.
   */
  private WSEditor editorAccess;

  /**
   * Save the content back to the original URL from where it was loaded using the internal support. 
   * Useful only when you provide an initial URL from which the component is loaded.
   */
  public void save() {
    getWSEditorAccess().save();
  }
  
  /**
   * Untitled files counter
   */
  private static int untitledFilesCounter = 0;

  /**
   * Sets the content to edit.
   * <p>This does not guarantee that the set content has been interpreted, you should set an
   * {@link AuthorComponentListener} and listen for <code>documentTypeChanged()</code> before using
   * the author extension actions.
   * 
   * @param url     URL to load, can be <code>null</code> if the reader is specified
   *                If no XML content reader is given, the URL will be used both to obtain the
   *                content and to solve relative references (eg: images).
   *                If the XML content reader is also given, the URL will only be used to solve
   *                relative references from the file.
   * @param reader  The reader.
   * 
   * @throws AuthorComponentException When there was a load problem (eg: IOException).
   */
  @Override
  public void load(URL url, Reader reader) throws AuthorComponentException{
    if(url != null) {
      PersistentEditor editorForURL = editor.getEditorManager().getEditorForURL(url);
      if(editorForURL != null && editor != editorForURL) {
        //Another author component provider has already been built over the URL and not properly disposed.
        throw new AuthorComponentException("Multiple component provider instances cannot be created over the same URL.\n" +
            "Another component provider has already been created for this URL.\n" +
            "Please dispose a component provider which is not necessary anymote using AuthorComponentFactory.disposeEditorComponentProvider().");
      }
    }
    
    final URL oldURL = editor.getEditorLocation();
    
    try {
      detectionFinished = false;
      editor.getDocumentTypeProvider().setDetectionTaskListener(new DetectionTaskListener() {
        @Override
        public void detectionTaskFinished() {
          detectionFinished = true;
        }
      });
      String reference = null;
      if (reader == null && url != null) {
        URLInfo info = URLUtil.getURLInfo(url);
        reference = info.getReference();
        //Show the file status
        statusPanel.setShowFilePath(true);
        Map<String, String> customQueryParameters = info.getCustomQueryParameters();
        if (customQueryParameters != null) {
          editor.setCustomQueryParams(customQueryParameters);
        }
        editor.open(info.getURL(true, false, true));
        //Request focus in the opened editor
        editor.requestFocusInEditor();
      } else if (reader != null) {
        if (url != null) {
          URLInfo info = URLUtil.getURLInfo(url);
          reference = info.getReference();
          Map<String, String> customQueryParameters = info.getCustomQueryParameters();
          if (customQueryParameters != null) {
            editor.setCustomQueryParams(customQueryParameters);
          }
          url = info.getURL(true, false, true);
        }
        //Do not show the file status
        statusPanel.setShowFilePath(false);
        boolean hasInitialURL = url != null;
        if (url == null) {
          url = URLUtil.correct(new File("Untitled" + (++untitledFilesCounter) + ".xml"));
        }
        
        editor.reloadFromReader(reader, url);
        if(hasInitialURL) {
          //EXM-26899 Do not consider it as untitled.
          editor.setFileEverSaved(true);
        }
        //Request focus in the opened editor
        editor.requestFocusInEditor();
      } else {
        throw new AuthorComponentException("Both URL and Reader are NULL");
      }
      
      // EXM-25439 Go to reference
      if (reference != null) {
        editor.goToReference(reference);
      }

      UiUtil.invokeSynchronously(new Runnable() {
        @Override
        public void run() {
          //Editor relocated.
          editor.getEditorManager().fireEditorRelocated(editor, oldURL);
        }
      });
      
      //Fire loaded document changed
      for (int i = 0; i < authorComponentListeners.size(); i++) {
        authorComponentListeners.get(i).loadedDocumentChanged();
      }
    } catch (Exception e) {
      e.printStackTrace();
      throw new AuthorComponentException(e);
    }
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.component.EditorComponentProvider#showLocation(java.net.URL, java.io.Reader)
   */
  @Override
  public void showLocation(URL url, Reader reader) throws AuthorComponentException {
    try {
      URL currentEditorLocation = editor.getEditorLocation();
      boolean loadURL = false;
      // Get the URL information
      URLInfo newEditorLocationinfo = URLUtil.getURLInfo(url);
      
      if (currentEditorLocation != null) {
        URL urlToFind = URLUtil.correct(url);
        urlToFind = URLUtil.getCanonicalURL(urlToFind);
        String key = URLUtil.getEditorLocationKey(urlToFind);
        if (Equaler.verifyEquals(key, URLUtil.getEditorLocationKey(currentEditorLocation))) {
          // The document is already loaded in the editor
          loadURL = false;
        } else {
          // Another document is loaded in the editor
          loadURL = true;
        }
      } else {
        // There is no document loaded in the editor
        loadURL = true;
      }

      if (loadURL) {
        load(url, reader);
      } else {
        // Go to reference
        if (newEditorLocationinfo.getReference() != null) {
          editor.goToReference(newEditorLocationinfo.getReference());
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
      throw new AuthorComponentException(e);
    }
  }
  
  /**
   * Constructor.
   *
   * @param applcetCodeBase     Applet location.
   * @param parentEditorManager The view manager.
   * @param parentFrame         View's parent frame.
   * @param allowedPages  The enumeration of allowed pages.
   * @param initialPage   The initial page.
   */
  AuthorComponentProvider(
      final URL appletCodeBase,
      final AuthorComponentEditorManager parentEditorManager,
      Frame parentFrame,
      final String[] allowedPages,
      final String initialPage) throws AuthorComponentException {
    if (logger.isDebugEnabled()) {
      logger.debug("Create author component");
    }
    if(allowedPages == null || allowedPages.length == 0) {
      throw new IllegalArgumentException("There are no allowed pages specified in the component");
    }
    editor = new XmlEditor(parentFrame, parentEditorManager, null, null, null) {
      /**
       * @see ro.sync.exml.editor.xmleditor.XmlEditor#createEditorPages()
       */
      @Override
      protected List<EditorPage> createEditorPages() {
        //Only Author page available for now.
        List<EditorPage> pages = new ArrayList();
        //Create the corresponding pages
        for (int i = 0; i < allowedPages.length; i++) { 
          String pageID = allowedPages[i];
          if(EditorPageConstants.PAGE_AUTHOR.equals(pageID)) {
            pages.add(new AuthorEditorPage(this));
          } else if(EditorPageConstants.PAGE_GRID.equals(pageID)) {
            pages.add(new GridEditorPage(this));
          } else if(EditorPageConstants.PAGE_TEXT.equals(pageID)) {
            pages.add(createTextPage());
          }
        }
        return pages;
      }
      /**
       * @see ro.sync.exml.editor.xmleditor.AbstractXmlEditor#getInitialPage()
       */
      @Override
      protected String getInitialPage() {
        return initialPage;
      }
      /**
       * @see ro.sync.exml.editor.AbstractEditor#setSelectedBorder(boolean)
       */
      @Override
      protected void setSelectedBorder(boolean isSelected) {
        if (isSampleApplet()) {
          // Do not put borders for sample editors.
          setBorder(null);
        } else {
          //Show border only if the license is for evaluation purposes
          LicenseInformation li = LicenseInformationProvider.getInstance().getLicenseInformation();
          if (li != null && li.isEvaluationOnly()) {
            super.setSelectedBorder(isSelected);
          }
        }
      }
      
      /**
       * Check if this is a sample applet from oxygenxml or devel-new.
       * 
       * @return <code>true</code> if this is a sample applet.
       */
      private boolean isSampleApplet() {
        boolean sampleApplet = false;
        String host = appletCodeBase != null ? appletCodeBase.getHost() : "";
        if (host.contains("devel-new.sync.ro") || host.contains("oxygenxml.com")) {
          sampleApplet = true;
        }
        return sampleApplet;
      }
    };
    editor.initFrame();
    //Do not use the lazy bounds.
    editor.setEnableSetBoundsInEditor(true);
    //Start from a dummy URL, avoid NPE if user does not set any content
    try {
      editor.setLoadedURL(URLUtil.correct(new File("Untitled.xml")));
    } catch (MalformedURLException e) {
      //
    }
    //No page switch yet
    if(allowedPages.length == 1) {
      editor.setShowPagesSwitch(false);
    }

    //Fire the component listeners
    editor.addEditorListener(
        new EditorListener() {
          @Override
          public void modifiedStateChanged(Descriptor editor) {
            for (int i = 0; i < authorComponentListeners.size(); i++) {
              authorComponentListeners.get(i).modifiedStateChanged(isModified());
            }
          }
          @Override
          public void loadedFileChanged(Descriptor editor) {
            //
          }
          @Override
          public void nameChanged(BasicEditor editor) {
            //
          }
          @Override
          public void editorPageChanged(EditorPageChangeEvent event) {
            //Only the Author editor page is used for now 
            //        for (int i = 0; i < authorComponentListeners.size(); i++) {
            //          authorComponentListeners.get(i).editorPageChanged();
            //        }
          }
          @Override
          public void editorLoaded(AbstractEditor editor) {
            //
          }
          @Override
          public void editorClosed(AbstractEditor editor) {
            // Not needed.
            if(statusPanel != null) {
              statusPanel.dispose();
            }
          }
        });
    
    
    //The editor access
    editorAccess = new WSEditorImpl(editor, editor, null);
    
    //Fire to the component listeners
    editor.getDocumentTypeProvider().addDTListener(
        new DocumentTypeProviderListener() {
          @Override
          public void documentTypeChanged(
              DocumentTypePO documentType, ExtensionsBundle bundle, long detectTime) {
            for (int i = 0; i < authorComponentListeners.size(); i++) {
              authorComponentListeners.get(i).documentTypeChanged();
            }
          }

          @Override
          public void cssChanged(CSSDescriptorPO[] cssDescriptors, long detectTime) {
            for (int i = 0; i < authorComponentListeners.size(); i++) {
              authorComponentListeners.get(i).documentTypeChanged();
            }
          }

          @Override
          public void authorExtensionChanged(
              AuthorExtensionPO authorExtension,
              ExtensionsBundle bundle,
              DocumentTypePO documentType,
              long detectionTime) {
            for (int i = 0; i < authorComponentListeners.size(); i++) {
              authorComponentListeners.get(i).documentTypeChanged();
            }
          }
        });
    editor.getStatusModel().addStatusModelListener(statusPanel);
    
    //Now register the keyboard shortcuts for these common actions
    addActionsToActionMap();
    editorAccess.addPageChangedListener(new WSEditorPageChangedListener() {
      @Override
      public void editorPageChanged() {
        addActionsToActionMap();
      }
    });
    
    //EXM-27030 Add the file drop manager.
    FileDropHandler fileDropHandler = new FileDropHandler();
    //The priority file drop manager listener
    fileDropHandler.addPriorityFileDropManagerListener(new EditorFileDropManagerListener(editor));
    editor.addDropHandler(fileDropHandler);

    //Fire the editor selected from the beginning. Useful to show the evaluation border
    editor.editorSelected(true, false);
  }

  /**
   * Add the Author actions to the action map.
   */
  private void addActionsToActionMap() {
    Map<String, Object> actions = null;
    if(EditorPageConstants.PAGE_AUTHOR.equals(editorAccess.getCurrentPageID())) {
     actions = ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getAuthorCommonActions();
    } else if(EditorPageConstants.PAGE_TEXT.equals(editorAccess.getCurrentPageID())) {
      actions = ((WSTextEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getTextActions();
     }
    if(actions != null) {
      Iterator<Object> iter = actions.values().iterator();
      while(iter.hasNext()) {
        AbstractAction action = (AbstractAction) iter.next();
        if (action instanceof AccelAction) {
          //And register them.
          if(EditorPageConstants.PAGE_AUTHOR.equals(editorAccess.getCurrentPageID())) {
            ((AccelAction)action).putInActionMap(getAuthorPage().getSwingComponent());
          } else {
            ((AccelAction)action).putInActionMap((JComponent) ((WSTextEditorPage)getWSEditorAccess().getCurrentPage()).getTextComponent());
          }
        }
      }
    }
  }

  /**
   * Create a reader over the editor's current page content
   * 
   * @return The reader over the current page's content
   */
  public Reader createReader() {
    return editor.createReader(); 
  }
  
  /**
   * Adds an author component listener.
   * 
   * @param listener The listener.
   */
  @Override
  public void addAuthorComponentListener(AuthorComponentListener listener) {
    if (!authorComponentListeners.contains(listener)) {
      authorComponentListeners.add(listener);
    }
  }
  
  /**
   * Removes an author component listener.
   * 
   * @param listener The listener.
   */
  @Override
  public void removeAuthorComponentListener(AuthorComponentListener listener) {
    authorComponentListeners.remove(listener);
  }  
  
  /**
   * @return The editor panel.
   */
  @Override
  public Component getEditorComponent() {
    return editor.getComponent();
  }
  
  /**
   * @return The status panel.
   */
  @Override
  public Component getStatusComponent() {
    return statusPanel;
  }
  
  /**
   * Gets the map of author extension actions.
   * 
   * <p>This should get called after each load as the extension actions depend on the loaded document type.
   * 
   * @return The map with (action id, AbstractAction) pairs with the actions defined in the Author framework.
   *         Can be <code>null</code> if no actions available.
   *         
   * @deprecated Please use instead the method ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getAuthorExtensionActions().
   */
  @Deprecated
  public Map<String, AbstractAction> getAuthorExtensionActions() {
    Map<String, Object> authorExtensionActions =
        ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getAuthorExtensionActions();
    //Radu C - Avoid NPE when requested for XML with no associated Document Type association
    if(authorExtensionActions != null) {
      //Convert to swing actions, which they already are.
      Map<String, AbstractAction> authActions = new LinkedHashMap<String, AbstractAction>();
      Iterator<String> iter = authorExtensionActions.keySet().iterator();
      while(iter.hasNext()) {
        String key = iter.next();
        Object val = authorExtensionActions.get(key);
        authActions.put(key, (AbstractAction) val);
      }
      return authActions;
    }
    return null;
  }
  
  /**
   * Get the map of author common actions (undo, redo, cut, copy, paste, etc).
   * 
   * @return The map with (action id, AbstractAction) pairs with the actions defined for working in
   *         the Author.
   *         
   * @deprecated Please use instead the method ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getAuthorCommonActions().
   */
  @Deprecated
  public Map<String, AbstractAction> getAuthorCommonActions(){
    Map<String, Object> authorCommonActions = ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getActionsProvider().getAuthorCommonActions();
    
    //Convert to swing actions, which they already are.
    Map<String, AbstractAction> authActions = new LinkedHashMap<String, AbstractAction>();
    Iterator<String> iter = authorCommonActions.keySet().iterator();
    while(iter.hasNext()) {
      String key = iter.next();
      Object val = authorCommonActions.get(key);
      authActions.put(key, (AbstractAction) val);
    }
    return authActions;
  }
  
  /**
   * Check if the component is modified
   * 
   * @return <code>true</code> if the component is modified.
   */
  public boolean isModified() {
    return getWSEditorAccess().isModified();
  }
  
  /**
   * Sets the modified status.
   * 
   * @param modified <code>true</code> to flag as modified.
   * 
   * @since 13
   */
  public void setModified(boolean modified) {
    getWSEditorAccess().setModified(modified);
  }
  
  /**
   * Get the author access used to perform various operations on the Author Page.
   *
   * @return The author access.
   * 
   * @deprecated Please use instead the method ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getAuthorAccess().
   */
  @Deprecated
  public AuthorAccess getAuthorAccess() {
    return ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage()).getAuthorAccess();
  }
  
  /**
   * Get the access to the WS Editor.
   *
   * @return The author access.
   */
  @Override
  public WSEditor getWSEditorAccess() {
    return editorAccess;
  }
  
  /**
   * @return The editor.
   */
  @Override
  protected Object getEditorKey() {
    return editor;
  }
  
  /**
   * @return The author page.
   */
  AuthorEditorPage getAuthorPage() {
    return ((AuthorEditorPage)editor.getEditorPage(EditorPage.PAGE_AUTHOR));
  }
  
  /**
   * The Pop-up customizer can be used to add/remove actions from the pop-up menu in the Author page
   * editor before showing it. If everything is removed then the menu will not be shown.
   * 
   * @param popUpCustomizer The pop Up Customizer.
   * 
   *  @deprecated Please use instead the method ((WSAuthorEditorPage)getWSEditorAccess().getCurrentPage())addPopUpMenuCustomizer(AuthorPopupMenuCustomizer popUpCustomizer)
   */
  @Deprecated
  public void setEditorPopUpCustomizer(PopupMenuCustomizer popUpCustomizer) {
    getAuthorPage().setEditorPopUpCustomizer(popUpCustomizer);
  }
  
  /**
   * The Pop-up customizer can be used to add/remove actions from the pop-up menu in the
   * Author Outliner view before showing it. If everything is removed then the menu will not be shown.
   * 
   * @param popUpCustomizer The pop Up Customizer.
   * 
   * @deprecated Please use instead the method ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).setOutlinerPopUpCustomizer();
   */
  @Deprecated
  public void setOutlinerPopUpCustomizer(PopupMenuCustomizer popUpCustomizer) {
    ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).setOutlinerPopUpCustomizer(popUpCustomizer);
  }
  
  /**
   * Show or hide the Bread Crumb panel in Author component.
   * 
   * @param showBreadCrumb <code>true</code> to show the Bread Crumb.
   * 
   * @since 14.1
   *
   * @deprecated Please use instead the method ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).showBreadCrumbPanel();
   */
  @Deprecated
  public void showBreadCrumb(boolean showBreadCrumb) {
    ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).showBreadCrumb(showBreadCrumb);
  }
  
  /**
   * The Pop-up customizer can be used to add/remove actions from the pop-up menu in the
   * Author bread crumb before showing it. If everything is removed then the menu will not be shown.
   * 
   * @param popUpCustomizer The pop Up Customizer.
   * 
   *  @deprecated Please use instead the method ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).setBreadCrumbPopUpCustomizer();
   */
  @Deprecated
  public void setBreadCrumbPopUpCustomizer(PopupMenuCustomizer popUpCustomizer) {
    ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).setBreadCrumbPopUpCustomizer(popUpCustomizer);
  }
  
  /**
   * Get an additional edit helper panel.
   * It can be the outline, attributes, entities, elements or model helper component, depending on the ID.
   * 
   * @param helperID One of:
   *                 <ul>
   *                  <li>AuthorComponentProvider.ATTRIBUTES_PANEL_ID,
   *                  <li>AuthorComponentProvider.ELEMENTS_PANEL_ID, 
   *                  <li>AuthorComponentProvider.ENTITIES_PANEL_ID,
   *                  <li>AuthorComponentProvider.MODEL_PANEL_ID,
   *                  <li>AuthorComponentProvider.OUTLINER_PANEL_ID constants.
   *                 </ul>
   *
   * @return The additional component.
   */
  @Override
  public JComponent getAdditionalEditHelper(int helperID){
    return new AdditionalEditHelperPanel(editor, helperID);
  }
  
  /**
   * Print the author component content. Shows the Print dialog.
   * 
   * @param preview <code>true</code> to show the Print Preview dialog, <code>false</code> to show the Print dialog.
   * 
   * @since 13
   */
  @Override
  public void print(boolean preview) {
    editor.print(preview);
  }
  
  /**
   * Create toolbars with all the actions defined at the framework level in exactly the same order in which they have been added to the toolbars
   * from the Document Type Edit dialog.
   * The toolbars will look almost identical with the ones which appear when the XML is opened in an Oxygen standalone version.
   * 
   * @return toolbars with all the actions defined at the framework level.
   *
   * @since 14.1 
   *
   * @deprecated Please use instead the method ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).createExtensionActionsToolbars();
   */
  @Deprecated
  public List<JToolBar> createExtensionActionsToolbars() {
    return ((WSAuthorComponentEditorPage)getWSEditorAccess().getCurrentPage()).createExtensionActionsToolbars();
  }

  /**
   * Show the location referenced by a given URL in the editor.
   * <br/>
   * If the document pointed by this URL is different than the document currently 
   * loaded in the editor page, this URL will be used to set the content to edit, 
   * to solve relative references (eg: images) and to show the location pointed by the
   * URL reference part. 
   * <br/>
   * If the document pointed by this URL is currently loaded in the editor page,
   * only the reference part of the given URL will be used to show the corresponding location
   * in the editor.  
   * 
   * @param url The URL to show location for. 
   *
   * @throws AuthorComponentException When there was a load problem (eg: IOException).
   * 
   * @since 14.1 
   */
  public void showLocation(URL url) throws AuthorComponentException {
    showLocation(url, null);
  }
}