/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.editor;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.view.graphics.Point;
import ro.sync.exml.view.graphics.Rectangle;

/**
 * It can be used when more than one editor types are needed depending on the received context and it can choose at runtime
 * an appropriate editor implementation.
 * For example if in the CSS oxy_editor() function we have a <b>test</b> parameter with more possible 
 * values and we must provide a different editor implementation for each one.
 *  
 * @since 14.1
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class AbstractInplaceEditorWrapper implements InplaceEditor {
  /**
   * The wrapped editor.
   */
  private InplaceEditor cEditor;

  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "A popup editor that depending on the singleSelection flag delegates to a different editor.";
  }
  
  /**
   * Depending of the received context, create an editor.
   * 
   * @param context Editing context.
   * 
   * @return An in-place editor.
   */
  protected abstract InplaceEditor createEditor(AuthorInplaceContext context);

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getEditorComponent(ro.sync.ecss.extensions.api.editor.AuthorInplaceContext, ro.sync.exml.view.graphics.Rectangle, ro.sync.exml.view.graphics.Point)
   */
  @Override
  public final Object getEditorComponent(AuthorInplaceContext context, Rectangle allocation,
      Point mouseInvocationLocation) {
    cEditor = createEditor(context);
    return cEditor.getEditorComponent(context, allocation, mouseInvocationLocation);
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getScrollRectangle()
   */
  @Override
  public final Rectangle getScrollRectangle() {
    return cEditor.getScrollRectangle();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#addEditingListener(ro.sync.ecss.extensions.api.editor.InplaceEditingListener)
   */
  @Override
  public final void addEditingListener(InplaceEditingListener editingListener) {
    cEditor.addEditingListener(editingListener);
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#requestFocus()
   */
  @Override
  public final void requestFocus() {
    cEditor.requestFocus();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#getValue()
   */
  @Override
  public final Object getValue() {
    return cEditor.getValue();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#stopEditing()
   */
  @Override
  public final void stopEditing() {
    cEditor.stopEditing();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#cancelEditing()
   */
  @Override
  public final void cancelEditing() {
    cEditor.cancelEditing();
  }

  /**
   * @see ro.sync.ecss.extensions.api.editor.InplaceEditor#removeEditingListener(ro.sync.ecss.extensions.api.editor.InplaceEditingListener)
   */
  @Override
  public final void removeEditingListener(InplaceEditingListener editingListener) {
    cEditor.removeEditingListener(editingListener);
  }
}