/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.view.graphics;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;

/**
 * Common font metrics
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PRIVATE)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.NOT_SPECIFIED, methods = SkipLevel.PUBLIC)
public interface FontMetrics {

  /**
   * Returns the total advance width for showing the specified array
   * of characters in this <code>Font</code>.  
   * 
   * @param chars the array of characters to be measured
   * @param off the start offset of the characters in the array
   * @param len the number of characters to be measured from the array
   * @return    the advance width of the subarray of the specified
   *               <code>char</code> array in the font described by
   *               this <code>FontMetrics</code> object.
   */
  int charsWidth(char[] chars, int off, int len);

  /**
   * The font ascent is the distance from the font's baseline to the top of most
   * alphanumeric characters.
   * 
   * @return The font ascent.
   */
  int getAscent();

  /**
   * The font descent is the distance
   * from the font's baseline to the bottom of most alphanumeric
   * characters with descenders.
   * @return The ont descent.
   */
  int getDescent();

  /**
   * Returns the total advance width for showing the specified 
     * <code>String</code> in this <code>Font</code>.
   * 
   * @param text The text.
   * @return The width of the text.
   */
  int stringWidth(String text);

  /**
   * @return The font height
   */
  int getHeight();

  /**
   * Get the char width
   * 
   * @param c The char
   * @return The char width
   */
  int charWidth(char c);
  
  /**
   * Determines the <em>standard leading</em> of the 
   * <code>Font</code> described by this <code>FontMetrics</code>
   * object.  The standard leading, or
   * interline spacing, is the logical amount of space to be reserved
   * between the descent of one line of text and the ascent of the next
   * line. The height metric is calculated to include this extra space.
   * @return    the standard leading of the <code>Font</code>.
   */
  int getLeading();
  
  /**
   * Gets the bounds of a string that is wrapped at the wrappingWidth. Takes into account newlines.
   * @param aStr The text, as an attributed string. The {TextAttribute#FONT} 
   * and {TextAttribute#FOREGROUND} contain values that are 
   * instances of {@link Font} and {@link Color}.
   * @param wrappingWidth The maximum width of the text.
   * @return The bounds of the wrapped string.
   */
  Rectangle getWrappedStringBounds(AttributedString aStr, int wrappingWidth);

  /**
   * Gets the y coordinate where the given wrapped string bounds can be split 
   * without cutting off a text line.
   * If the given initial break Y is inside a text line, the resulting break will
   * point to the top of the text line. Otherwise <code>-1</code> is returned.
   * 
   * 
   * @param aStr The text, as an attributed string. The {TextAttribute#FONT} 
   * and {TextAttribute#FOREGROUND} contain values that are 
   * instances of {@link Font} and {@link Color}.
   * @param wrappingWidth The maximum width of the text.
   * @param initialBreak The initial break Y coordinate.
   * @return The corrected Y break coordinate or <code>-1</code> if the initial 
   * break is not placed inside a text line.
   */
  int getCorrectedBreakYForWrappedString(AttributedString aStr, int wrappingWidth, int initialBreak);
  
}
