/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.editor.page.ditamap.keys;

import java.util.LinkedHashSet;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.util.Equaler;

/**
 * An enumeration def info.
 *
 * &lt;enumerationdef>
 *  &lt;elementdef name="p"/>
 *  &lt;attributedef name="product"/>
 *  &lt;subjectdef keyref="test"/>
 * &lt;/enumerationdef>
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
@SkipObfuscate(classes = SkipLevel.PRIVATE, fields = SkipLevel.PRIVATE, methods = SkipLevel.PRIVATE)
public class EnumerationDefInfo {
    
  /**
   * The attribute def name.
   */
  private String attributeName;
  
  /**
   * The element def name.
   */
  private String elementName;
  
  /**
   * The set of referenced keys.
   */
  private LinkedHashSet<String> referencedKeys;
  
  /**
   * Constructor.
   * 
   * @param attributeName The attribute name.
   * @param elementName The element name. Can be <code>null</code>.
   */
  public EnumerationDefInfo(String attributeName, String elementName) {
    this.attributeName = attributeName;
    this.elementName = elementName;
  }
  
  /**
   * Get the attribute name.
   * 
   * @return Returns the attribute name.
   */
  public String getAttributeName() {
    return attributeName;
  }
  
  /**
   * Get the element name.
   * 
   * @return Returns the element name.
   */
  public String getElementName() {
    return elementName;
  }
  
  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return "Elem name " + elementName + " Attr name:" + attributeName;
  }
  
  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if(obj instanceof EnumerationDefInfo) {
      EnumerationDefInfo other = (EnumerationDefInfo) obj;
      if(Equaler.verifyEquals(elementName, other.elementName)
          && Equaler.verifyEquals(attributeName, other.attributeName)) {
        return true;
      }
    }
    return false;
  }
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    return (elementName + "@" + attributeName).hashCode();
  }
  
  /**
   * Get the set of referenced keys, can be <code>null</code>.
   * 
   * @return Returns the referenced key names.
   */
  public LinkedHashSet<String> getReferencedKeys() {
    return referencedKeys;
  }
  
  /**
   * Add a referenced key.
   * @param keyRef The keyref.
   */
  public void addReferencedKey(String keyRef) {
    if(referencedKeys == null) {
      referencedKeys = new LinkedHashSet<String>();
    }
    referencedKeys.add(keyRef);
  }
  
}
